//
//  AMSDKMeetingClient.h
//  AliMeetingSDK
//
//  Created by xuanzhen.zjw on 2019/8/15.
//  Copyright © 2019 Alibaba. All rights reserved.
//

#import <UIKit/UIKit.h>

/**
 * 加入会议设备类型枚举
 */
typedef NS_ENUM(NSInteger, AMSDKClientType) {
    //! MCU Client
    AMSDKClientType_MCU             =  0,
    //! 屏幕分享流
    AMSDKClientType_SCREEN_SHARE    =  1,
    //! 普通Client，关联人员的Client
    AMSDKClientType_USER            =  2,
    //! 普通Client，关联人员的Client
    AMSDKClientType_ROOM            =  3
};

/**
 * 网络质量枚举
 * 共分六级，与信令服务器给的值对应，具体是否弱网还需要根据服务端返回判断
 */
typedef NS_ENUM(NSInteger, AMSDKNetworkQuality) {
    //! audio only
    AMSDKNetworkQuality_GRADE_ZERO  = 0,
    //! 有视频，但是效果很差
    AMSDKNetworkQuality_GRADE_ONE   = 1,
    //! 有视频，效果差
    AMSDKNetworkQuality_GRADE_TWO   = 2,
    //! 有视频，效果一般
    AMSDKNetworkQuality_GRADE_THREE = 3,
    //! 有视频，效果较好
    AMSDKNetworkQuality_GRADE_FOUR  = 4,
    //! 有视频，HD
    AMSDKNetworkQuality_GRADE_FIVE  = 5
};

/**
 * 网络状态
 */
typedef NS_ENUM(NSInteger, AMSDKNetworkStatus) {
    //! 网络已断开
    AMSDKNetworkStatus_DISCONNECT   = 0,
    //! 弱网状态
    AMSDKNetworkStatus_WEAK         = 1,
    //! 网络正常
    AMSDKNetworkStatus_NORMAL       = 2
};

/**
 * 加入会议设备类型枚举
 */
typedef NS_ENUM(NSInteger, AMSDKCallType) {
    //! 手机
    AMSDKCallType_CELL_PHONE          =  0,
    //! 分机
    AMSDKCallType_EXTENSION_PHONE     =  1
};

@class AMSDKRenderView;

NS_ASSUME_NONNULL_BEGIN

//! 操作回调
//! errorCode 错误码见@AMSDKErrorCode
//! error 错误信息
typedef void (^AMSDKActionCallback)(BOOL success, NSError * _Nullable error);

/**
 * AMSDKMeetingClient 调用线程安全
 * 用于抽象会议中的一个RTC端，RTC端包含音视频状态获取、共享状态获取、上线离线状态、说话和主讲人状态、视频渲染等。
 */
@interface AMSDKMeetingClient : NSObject

//! 一个RTC端的唯一标识，和@AMSDKUser的memberUUID相同
@property (nonatomic, copy, readonly) NSString *uuid;

//! 用户ID，clientType类型为AMSDKClientType_USER时有效
@property (nonatomic, copy, readonly) NSString *userId;

//! 姓名
@property (nonatomic, copy, readonly) NSString *name;

//! 端类型，见@AMSDKClientType
@property (nonatomic, assign, readonly) AMSDKClientType clientType;

//! 是否为会议发起人
@property (nonatomic, assign, readonly) BOOL isMaster;

//! 是否在响铃
@property (nonatomic, assign, readonly) BOOL isRinging;

//! 是否为当前入会方
@property (nonatomic, assign, readonly) BOOL isPublisher;

//! 是否为主讲人
@property (nonatomic, assign, readonly) BOOL isMainSpeaker;

//! 是否在说话中
@property (nonatomic, assign, readonly) BOOL isTalking;

//! 是否接通（在线）
@property (nonatomic, assign, readonly) BOOL isOnline;

//! 声音是否打开
@property (nonatomic, assign, readonly) BOOL isAudioOn;

//! 视频是否打开
@property (nonatomic, assign, readonly) BOOL isVideoOn;

//! 服务端视频流状态，与 isVideoOn 的区别在于 isVideoStreamEnabled 只判断服务端状态，而 isVideoOn会结合本地enableVideo状态进行综合判断
@property (nonatomic, assign, readonly) BOOL isVideoStreamEnabled;

//! 媒体流是否就绪
@property (nonatomic, assign, readonly) BOOL isStreamReady;

//! 网络质量，见@AMSDKNetworkQuality
@property (nonatomic, assign, readonly) AMSDKNetworkQuality networkQuality;

//! 网络状态，见@AMSDKNetworkStatus
@property (nonatomic, assign, readonly) AMSDKNetworkStatus networkStatus;

//! 入会时间戳，单位毫秒
@property (nonatomic, assign, readonly) long long joinTime;

//! 附加数据
@property (nonatomic, strong) id extraData;

//! 获取属性
- (NSString *)getAttribute:(NSString *)key;

//! 禁用音频，会影响其他端之间的表现，比如mute成员A，则会除了当前入会端外，其他入会到的看到的A也会被禁音
- (void)muteAudio:(BOOL)muted callback:(AMSDKActionCallback _Nullable)callback;

//! 禁用视频，会影响其他端之间的表现，比如mute成员A，则会除了当前入会端外，其他入会到的看到的A也会被禁视频
- (void)muteVideo:(BOOL)muted callback:(AMSDKActionCallback _Nullable)callback;

//! 对于publisher，该方法会调用muteVideo；对于非publisher成员，enabled == YES 接收对应client的视频流，enabled == NO 不接收对应client的视频流，视频流是否可用通过isVideoOn判断
- (void)enableVideo:(BOOL)enabled callback:(AMSDKActionCallback _Nullable)callback;

//! 发起呼叫
- (void)call:(AMSDKCallType)type callback:(AMSDKActionCallback _Nullable)callback;

//! 挂断
- (void)hangup:(AMSDKActionCallback _Nullable)callback;

//! 添加渲染视图
- (void)addRenderView:(AMSDKRenderView *)renderView;

//! 移除渲染视图
- (void)removeRenderView:(AMSDKRenderView *)renderView;

//! 移除所有渲染视图
- (void)removeAllRenderViews;

#pragma mark - 注意：下列初始化方法不可用

+ (instancetype)new NS_UNAVAILABLE;

- (instancetype)init NS_UNAVAILABLE;

@end

NS_ASSUME_NONNULL_END
